//////////////////////////////////////////////////////////////////////////
//
// FILE:   Interpolate.h
//
// DATE:   January 3, 1996
//
// AUTHOR:
//
//    James Kuffner, Jr
//    Computer Science Department
//    Stanford University
//    U.S.A.
//    email: kuffner@stanford.edu
//    http://robotics.stanford.edu/~kuffner
//
// DISTRIBUTION NOTICE
//
// Copyright (c) 1996 James Kuffner, Jr. All rights reserved.
//
// Permission to use, copy, modify and distribute this software for any
// purpose is hereby granted without fee, provided that the above
// copyright notice and this permission notice appear in all copies of
// this software and that you do not sell the software.
//
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef INTERPOLATE_H
#define INTERPOLATE_H

#ifndef IGNORE_DEPEND
#include <Xm/Xm.h>
#endif
#include "TextArea.h"

class Interface;
class Renderer;

//////////////////////////////////////////////////////////////////////
// Interpolate 
//////////////////////////////////////////////////////////////////////
class Interpolate {
 public:
  Interpolate(Widget mainWindow);
  ~Interpolate();
  
  // other methods relating to the user-interface
  void   ShowErrorDialog(char *s);
  void   AdvanceFrame();
  void   DrawFrame();
  void   UpdateValues(RotMode mode);
  void   SetNumViews(int n);
  void   SetKeyframe(int n);
  void   Playback(bool start);

 private:

  // callbacks for the text field editor widget
  static void _WeightChangeCB(TextArea *, float *val, TaPointer cd);
  static void _QuatChangeCB(TextArea *, float *val, TaPointer cd);
  static void _AxisChangeCB(TextArea *, float *val, TaPointer cd);
  static void _EulerChangeCB(TextArea *, float *val, TaPointer cd);
  static void _UpdateValuesCB(TextArea *, const float *val, TaPointer cd);


  // the GUI
  Widget       _mainWindow, _pane;
  Renderer     *_mfr;
  TextArea     *_textArea;
  Interface    *_interface;

  // data relating to playback
  bool         _stopped;
  float        _weight;

  // the key-frame data
  float _key1[4];
  float _key2[4];
  bool  _k1set;
  bool  _k2set;
};

#endif  // INTERPOLATE_H




