//////////////////////////////////////////////////////////////////////////
//
// FILE:   Renderer.h
//
// DATE:   January 3, 1996
//
// AUTHOR:
//
//    James Kuffner, Jr
//    Computer Science Department
//    Stanford University
//    U.S.A.
//    email: kuffner@stanford.edu
//    http://robotics.stanford.edu/~kuffner
//
// DISTRIBUTION NOTICE
//
// Copyright (c) 1996 James Kuffner, Jr. All rights reserved.
//
// Permission to use, copy, modify and distribute this software for any
// purpose is hereby granted without fee, provided that the above
// copyright notice and this permission notice appear in all copies of
// this software and that you do not sell the software.
//
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef RENDERER_H
#define RENDERER_H

#ifndef IGNORE_DEPEND
#include <Xm/Xm.h>
#endif

#define  NUM_VIEWERS  4

//inventor classes
class SoSeparator;
class SoSensor;
class SoXtExaminerViewer;
class SoTimerSensor;
class SoTransform;
class SoSelection;
class SoSwitch;

// my classes
class Interpolate;


class Renderer {
 public:
  Renderer(Widget parent, Interpolate *interpolate);
  virtual ~Renderer();

  // specific methods
  void SetQuatTransform(const float q[4]);
  void GetQuatTransform(float q[4]);
  void SetAxisTransform(const float q[4]);
  void GetAxisTransform(float q[4]);
  void SetEulerTransform(const float q[3]);
  void GetEulerTransform(float q[3]);

  // general methods
  void SetNumViews(int numViews, bool changeView); //set the number of views
  void StartPlayback();   // schedule the timer
  void StopPlayback();
  void ToggleDisplay(int whichToggle);
  void AddSceneRoot(SoSeparator *sceneRoot);
  void RemoveSceneRoot(SoSeparator *sceneRoot);

  //the kinds of toggle switches
  enum { TOGGLE_GRID = 0, NUM_TOGGLES };


 private:

 // Methods

  //callback routine for timer interrupt and playback
  static void _PlayCB(void *userData, SoSensor *);
  //callback for resizing the rendering area
  static void _ResizeRenderAreaCB(Widget, void *userData, XtPointer);

  //Initialization routines. 
  SoSeparator* _ReadFile(const char *filename);
  void _InitGrid();
  void _InitAxes();
  void _InitPlayback();
  void _InitModel();


 //Data

  // the render area and inventor viewer wigets
  Interpolate        *_interpolate;
  Widget              _renderArea;
  SoXtExaminerViewer *_viewer[NUM_VIEWERS];  //the viewers
  int                 _numViews;    //the number of views displayed
  int                 _currentView;

  //flags for the toggle switch
  bool          _displayGrid;

  //Inventor Data
  SoTimerSensor *_timerSensor;     //used for animation callback
  SoSelection   *_ivRoot;          //root node of the inventor heirarchy
  SoSwitch      *_ivGridSwitch;    //switch node for grid display
  SoSeparator   *_ivGrid;          //node containing grid information

  //the model in the scene
  SoTransform   *_ivTransform; //transform node for the model
};

#endif  // RENDER_H
