//////////////////////////////////////////////////////////////////////////
//
// FILE:   TextArea.h
//
// DATE:   January 3, 1996
//
// AUTHOR:
//
//    James Kuffner, Jr
//    Computer Science Department
//    Stanford University
//    U.S.A.
//    email: kuffner@stanford.edu
//    http://robotics.stanford.edu/~kuffner
//
// DISTRIBUTION NOTICE
//
// Copyright (c) 1996 James Kuffner, Jr. All rights reserved.
//
// Permission to use, copy, modify and distribute this software for any
// purpose is hereby granted without fee, provided that the above
// copyright notice and this permission notice appear in all copies of
// this software and that you do not sell the software.
//
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef TEXTAREA_H
#define TEXTAREA_H

#ifndef IGNORE_DEPEND
#include <Xm/Xm.h>
#endif

////////////////
// Callback types which can be registered with the text area.

enum TaCallbackType {
  TaQuatChange, TaAxisChange, TaEulerChange, TaUpdateValues,
  TaWeightChange, TaNumCallbacks
};

enum {
  P_1=0, P_2, P_3, P_4, P_WEIGHT, NUM_FIELDS
};

typedef enum {
  QUAT=0, AXIS, EULER, NUM_RADIO_BUTTONS
} RotMode;


////////////////
// Type for a callback function.

class TextArea;
typedef void *TaPointer;

typedef int (*TaCallback)(TextArea *ta, const float *val,
			  TaPointer clientData); 

////////////////
// Structure to describe the text area

class TextArea {
 public:
  TextArea(Widget parent);
  ~TextArea() {}
  void setCallback(TaCallbackType type, TaCallback callback,
		   TaPointer clientData = NULL);
  int executeCallback(TaCallbackType type);
  void SetSliderValues(float q[4]);
  void GetSliderValues(float q[4]);
  void SetWeight(float weight);
  void InitMode();
  RotMode Mode() { return _mode; }

 private:
  void _InitInterface(Widget parent);
  void _UpdateMode();

 //Callbacks
  static void _RadioCB(Widget w, XtPointer clientData, XtPointer callData);
  static void _TextCB(Widget w, XtPointer clientData, XtPointer callData);

 //Data
  Widget     _parent, _mainWindow;
  Widget     _edit[NUM_FIELDS], _label[NUM_FIELDS];
  Widget     _radWidget[NUM_RADIO_BUTTONS];
  float      _values[NUM_FIELDS];
  RotMode    _mode;

  TaCallback _callbacks[TaNumCallbacks];
  TaPointer  _clientData[TaNumCallbacks];
};
#endif //TEXTAREA_H
